﻿#region Using Statements
using System.Collections.Generic;
using RhinoXNA.RhinoUtilities;
using RhinoXNA;
using RhinoXNA.RhinoGraphics.Rhino2D;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
#endregion

namespace AsteroidsGame.Game_Objects
{
    /// <summary>
    /// Singleton instance of the Asteroid Factory
    /// </summary>
    public sealed class AsteroidFactory
    {
        #region Private Variables
        // List of asteroids
        private List<Asteroid> _asteroids;

        // Singleton stuff
        private static volatile AsteroidFactory _manager;
        private static object syncRoot = new Object();

        // Saved parameters to generate asteroids
        private List<String> _asteroidParams;
        #endregion

        #region Private Methods
        /// <summary>
        /// Creates a new instance of this class type
        /// </summary>
        private AsteroidFactory()
        {
            _asteroids = new List<Asteroid>();
        }

        /// <summary>
        /// Generates a new randomized asteroid
        /// </summary>
        private void GenerateNewAsteroid()
        {
            _asteroidParams = new List<String>()
            {
                "Asteroid",
                "Sprites\\asteroid",
                "asteroid" + _asteroids.Count.ToString(), 
                "0", "0",
                "0", 
                "0.5", "0.5", 
                "0.5", "true", 
                "0", "0", "WIDTH", "HEIGHT"
            };

            _asteroids.Add(new Asteroid(_asteroidParams, true));
        }
        #endregion

        #region Public Properties
        /// <summary>
        /// Returns the current list of asteroids
        /// </summary>
        public List<Asteroid> Asteroids
        {
            get { return _asteroids; }
        }

        /// <summary>
        /// Returns the current instance of the Asteroid Factory
        /// </summary>
        public static AsteroidFactory Instance
        {
            get
            {
                if (_manager == null)
                {
                    lock (syncRoot)
                    {
                        if (_manager == null)
                        {
                            _manager = new AsteroidFactory();
                        }
                    }
                }

                return _manager;
            }
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Creates n-count of asteroids
        /// </summary>
        /// <param name="asteroidCount">Number of asteroids to generate</param>
        public void GenerateDefaultAsteroids(int asteroidCount)
        {
            for (int i = 0; i < asteroidCount; i++)
            {
                GenerateNewAsteroid();
            }
        }

        /// <summary>
        /// Updates all existing asteroids
        /// </summary>
        /// <param name="gameTime">Current gametime</param>
        public void Update(GameTime gameTime)
        {
            for (int i = 0; i < _asteroids.Count; i++)
            {
                _asteroids[i].Update(gameTime);

                List<Asteroid> collided = _asteroids.FindAll(delegate(Asteroid a)
                {
                    return (a.Handle != _asteroids[i].Handle) 
                        && (_asteroids[i].BasicBoundingBox.Intersects(a.BasicBoundingBox));
                });

                if (collided.Contains(_asteroids[i].LastCollision) == false)
                {
                    _asteroids[i].LastCollision = null;
                }

                for (int j = 0; j < collided.Count; j++)
                {
                    Vector2 collisionAmount = Vector2.Zero;

                    if (_asteroids[i].Position.X <= collided[j].Position.X)
                    {
                        if (_asteroids[i].Position.Y <= collided[j].Position.Y)
                        {
                            collisionAmount = new Vector2(
                                 Math.Abs(_asteroids[i].BasicBoundingBox.Right - collided[j].BasicBoundingBox.Left),
                                 Math.Abs(_asteroids[i].BasicBoundingBox.Bottom - collided[j].BasicBoundingBox.Top)
                             );
                        }
                        else
                        {
                            collisionAmount = new Vector2(
                                 Math.Abs(_asteroids[i].BasicBoundingBox.Right - collided[j].BasicBoundingBox.Left),
                                 Math.Abs(_asteroids[i].BasicBoundingBox.Top - collided[j].BasicBoundingBox.Bottom)
                             );
                        }
                    }
                    else
                    {
                        if (_asteroids[i].Position.Y <= collided[j].Position.Y)
                        {
                            collisionAmount = new Vector2(
                                 Math.Abs(_asteroids[i].BasicBoundingBox.Left - collided[j].BasicBoundingBox.Right),
                                 Math.Abs(_asteroids[i].BasicBoundingBox.Bottom - collided[j].BasicBoundingBox.Top)
                             );
                        }
                        else
                        {
                            collisionAmount = new Vector2(
                                 Math.Abs(_asteroids[i].BasicBoundingBox.Left - collided[j].BasicBoundingBox.Right),
                                 Math.Abs(_asteroids[i].BasicBoundingBox.Top - collided[j].BasicBoundingBox.Bottom)
                             );
                        }
                    }

                    if (collisionAmount.X >= collisionAmount.Y)
                    {
                        _asteroids[i].MovementDirection = new Vector2(_asteroids[i].MovementDirection.X, -_asteroids[i].MovementDirection.Y);
                        collided[j].MovementDirection = new Vector2(collided[j].MovementDirection.X, -collided[j].MovementDirection.Y);
                    }
                    else if (collisionAmount.X < collisionAmount.Y)
                    {
                        _asteroids[i].MovementDirection = new Vector2(-_asteroids[i].MovementDirection.X, _asteroids[i].MovementDirection.Y);
                        collided[j].MovementDirection = new Vector2(-collided[j].MovementDirection.X, collided[j].MovementDirection.Y);
                    }

                    _asteroids[i].LastCollision = collided[j];
                }
            }
        }

        /// <summary>
        /// Renders all existing asteroids
        /// </summary>
        public void Render()
        {
            foreach (Asteroid a in _asteroids)
            {
                a.Render();
            }
        }

        /// <summary>
        /// Splits a given asteroid into 2 smaller asteroids
        /// </summary>
        /// <param name="asteroid">Asteroid to split</param>
        public void SplitAsteroid(Asteroid asteroid)
        {
            if (asteroid.MaxHealth > 1)
            {
                //Create 2 new, smaller asteroids
                for (int i = 0; i < 2; i++)
                {
                    int num = 1;
                    if (i == 1)
                    {
                        num = -1;
                    }

                    _asteroidParams = new List<String>()
                    {
                        "Asteroid",
                        "Sprites\\asteroid",
                        "asteroid" + _asteroids.Count.ToString(), 
                        asteroid.Position.X.ToString(),
                        asteroid.Position.Y.ToString(),
                        "0",
                        (asteroid.Scale.X/2).ToString(),
                        (asteroid.Scale.Y/2).ToString(),
                        "0.5", "true", 
                        "0", "0", "WIDTH", "HEIGHT",
                        (asteroid.MaxHealth/2).ToString(),
                        (num * asteroid.MovementDirection.X).ToString(),
                        (asteroid.MovementDirection.Y).ToString(),
                        num.ToString()
                    };
                    _asteroids.Add(new Asteroid(_asteroidParams, false));
                }
            }

            _asteroids.Remove(asteroid);
        }

        /// <summary>
        /// Removes all asteroids, resets asteroid count
        /// </summary>
        public void Clear()
        {
            _asteroids.Clear();
        }
        #endregion
    }
}
