﻿#region Using Statements
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using RhinoXNA.RhinoUtilities;
#endregion

namespace RhinoXNA.RhinoGraphics.Rhino2D
{
    /// <summary>
    /// Abstract class to provide lots of similar 2D functionality
    /// </summary>
    public abstract class SpriteEntity : IRhinoDrawable
    {
        #region Public Properties
        /// <summary>
        /// Returns the Source Rectangle of this sprite
        /// </summary>
        public Rectangle SourceRect { get; set; }

        /// <summary>
        /// Returns the basic bounding box for this sprite
        /// </summary>
        public Rectangle BasicBoundingBox
        {
            get
            {
                return new Rectangle((int)(this.Position.X - (this.Width / 2)),
                    (int)(this.Position.Y - (this.Height/2)),
                    (int)(this.Width * .9),
                    (int)(this.Height * .9));
            }
        }

        /// <summary>
        /// Scaled dimensions of this sprite
        /// </summary>
        public Vector2 Dimensions
        {
            get { return new Vector2(this.Width, this.Height); }
        }

        /// <summary>
        /// Returns the scaled height of this sprite
        /// </summary>
        public int Height
        {
            get { return (int)(_tex.Height * Scale.Y); }
        }

        /// <summary>
        /// Returns the scaled width of this sprite
        /// </summary>
        public int Width
        {
            get { return (int)(_tex.Width * Scale.X); }
        }

        /// <summary>
        /// Returns the 2D texture that defines this sprite
        /// </summary>
        public Texture2D Texture
        {
            get { return _tex; }
        }
        #endregion

        #region Private Variables
        // Stored sprite texture
        private Texture2D _tex;
        #endregion

        #region Public Methods
        /// <summary>
        /// Creates a new Sprite entity with the specified parameters
        /// </summary>
        /// <param name="parameters">Parameters to create the sprite</param>
        public SpriteEntity(List<String> parameters)
            : base ()
        {
            bool vis = false;
            if (parameters[9].ToLower() == "true")
                vis = true;

            _tex = RhinoXNAAdapter.Instance.ContentManager.Load<Texture2D>(parameters[1]);
            this.Position = RhinoLoader.GetPosition(_tex, parameters[3], parameters[4]);
            this.Rotation = MathHelper.ToRadians(Convert.ToInt32(parameters[5]));
            this.Scale = new Vector2(float.Parse(parameters[6]), float.Parse(parameters[7]));
            this.Layer = float.Parse(parameters[8]);
            this.SourceRect = new Rectangle(Convert.ToInt32(parameters[10]), Convert.ToInt32(parameters[11]),
                    Convert.ToInt32(RhinoLoader.GetSourceRectSize(_tex, parameters[12], parameters[13]).X),
                    Convert.ToInt32(RhinoLoader.GetSourceRectSize(_tex, parameters[12], parameters[13]).Y));
            this.isVisible = vis;
            this.Handle = parameters[2];
        }

        /// <summary>
        /// Creates a new sprite entity
        /// </summary>
        /// <param name="texture">Texture to define the sprite</param>
        /// <param name="handle">Handle to uniquely access the sprite</param>
        /// <param name="src">Source rectangle</param>
        /// <param name="pos">Screen position</param>
        /// <param name="rot">Rotation (in radians)</param>
        /// <param name="scale">Sprite scaling</param>
        /// <param name="isVisible">Whether the sprite is visible (should be drawn) or not</param>
        /// <param name="layer">Layer to draw the sprite on</param>
        public SpriteEntity(Texture2D texture, String handle, Rectangle src, Vector2 pos, float rot, Vector2 scale, bool isVisible, float layer)
            : base()
        {
            _tex = texture;
            this.Position = pos;
            this.Rotation = rot;
            this.Scale = scale;
            this.Layer = layer;
            this.SourceRect = src;
            this.isVisible = isVisible;
            this.Handle = handle;
        }

        /// <summary>
        /// Updates the sprite (calls Animate)
        /// </summary>
        /// <param name="gameTime">Current gametime</param>
        public override void Update(GameTime gameTime)
        {
            Animate(gameTime);
        }

        /// <summary>
        /// Renders the current Sprite
        /// </summary>
        public override void Render()
        {
            if (this.isVisible)
            {
                SpriteBatch batch = RhinoXNAAdapter.Instance.SpriteBatch;
                batch.Begin();
                batch.Draw(_tex, this.Position, this.SourceRect,
                    Color.White, this.Rotation, new Vector2(this.SourceRect.Width / 2, this.SourceRect.Height / 2), this.Scale,
                    SpriteEffects.None, this.Layer);
                batch.End();
            }
        }
        #endregion

        #region Protected Methods
        /// <summary>
        /// Animates the sprites during update
        /// </summary>
        /// <param name="gameTime">Current gametime</param>
        protected abstract void Animate(GameTime gameTime);
        #endregion
    }
}
